var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import React, { forwardRef, useEffect, useImperativeHandle, useRef, useState } from 'react';
import { ValidationEventType } from '../models/validation/ValidationEventType';
import { ValidationOverride } from '../models/validation/ValidationOverride';
import { ValidationResult } from '../models/validation/ValidationResult';
import { formValidator } from './helpers/FormValidator';
import { formbuilder } from '../utils/FormBuilderInitializer';
import { validationUtil } from '../utils/common/ValidationUtil';
import { schemaFromConfig } from '../utils/schema/schemaFromConfig';
import { useIsMounted } from '../hooks/useIsMounted';
const EmptyGroupContainer = props => {
    return React.createElement("div", { key: `emptygroup-${props.itemId}` }, props.children);
};
export const Form = forwardRef((props, ref) => {
    var _a, _b, _c;
    // verify that the formbuilder is set up correctly
    formbuilder.verify();
    const isMounted = useIsMounted();
    const [item, setItem] = useState(props.item);
    const [schema, setSchema] = useState(props.dynamicSchema != null ? undefined : props.schema);
    const dynamicSchema = useRef();
    const [groupings, setGroupings] = useState();
    const [groupsHandled, setGroupsHandled] = useState(false);
    const [validationResults, setValidationResults] = useState((_a = props.validationResult) !== null && _a !== void 0 ? _a : {});
    const validateItem = (formItem, schema, on = ValidationEventType.Manual, prop) => __awaiter(void 0, void 0, void 0, function* () {
        var _d;
        let selectedSchema = (_d = dynamicSchema.current) !== null && _d !== void 0 ? _d : schema;
        switch (props.validationOverride) {
            case undefined:
            case ValidationOverride.None:
                let results = yield formValidator.validate(formItem, prop, selectedSchema, on, validationResults, props.validationResultPrefix);
                setValidationResults(Object.assign({}, results));
                return validationUtil.validated(results) ? ValidationResult.Success : ValidationResult.Failed;
            case ValidationOverride.Ignore:
                setValidationResults({});
                return ValidationResult.Success;
            case ValidationOverride.Continue:
                let results2 = yield formValidator.validate(formItem, prop, selectedSchema, on, validationResults, props.validationResultPrefix);
                setValidationResults(Object.assign({}, results2));
                return validationUtil.validated(results2) ? ValidationResult.Success : ValidationResult.FailedDontBlock;
            default:
                return ValidationResult.Success;
        }
    });
    useImperativeHandle(ref, () => ({
        getItem: () => item,
        validateItem: (type) => validateItem(item, props.schema, type, undefined)
    }), [item, props.schema]);
    const getDynamicSchemaKey = (it) => {
        var _a;
        let dynamicKey = (_a = props.dynamicSchema) === null || _a === void 0 ? void 0 : _a.dynamicKey;
        if (dynamicKey == null)
            return undefined;
        return dynamicKey(it !== null && it !== void 0 ? it : item);
    };
    const loadSchema = (it, force) => __awaiter(void 0, void 0, void 0, function* () {
        var _e, _f, _g, _h;
        if (schema !== undefined && ((_e = props.dynamicSchema) === null || _e === void 0 ? void 0 : _e.dynamicKey) == null && !force)
            return;
        // let schemaMerged = await fetchSchema<T>(props.dynamicSchema?.customFormType ?? props.itemType, props.formItemConfigOverrides, props.propertyOverrides, getDynamicSchemaKey(it));
        let schemaMerged = yield schemaFromConfig((_g = (_f = props.dynamicSchema) === null || _f === void 0 ? void 0 : _f.schemaConfig) !== null && _g !== void 0 ? _g : props.schemaConfig, props.formItemConfigOverrides, props.propertyOverrides, getDynamicSchemaKey(it));
        if (((_h = props.dynamicSchema) === null || _h === void 0 ? void 0 : _h.schemaConfig) != null) {
            dynamicSchema.current = schemaMerged;
        }
        if (!isMounted())
            return;
        setSchema(schemaMerged);
    });
    useEffect(() => {
        loadSchema();
    }, [props.schemaConfig]);
    useEffect(() => {
        // if (props.propertyOverrides == null) return;
        loadSchema(undefined, true);
    }, [props.propertyOverrides]);
    const GroupContainer = (_b = props.groupContainer) !== null && _b !== void 0 ? _b : EmptyGroupContainer;
    useEffect(() => {
        buildGroupings();
    }, [schema, item, validationResults]);
    const buildGroupings = () => {
        var _a, _b;
        if (((_a = schema === null || schema === void 0 ? void 0 : schema.options) === null || _a === void 0 ? void 0 : _a.groups) == null) {
            setGroupsHandled(true);
            return;
        }
        let groups = (_b = schema === null || schema === void 0 ? void 0 : schema.options) === null || _b === void 0 ? void 0 : _b.groups;
        let groupedProps = {};
        Object.keys(schema.options.properties).forEach(_ => {
            var _a, _b;
            let prop = schema.options.properties[_];
            if ((_a = prop.hide) === null || _a === void 0 ? void 0 : _a.call(prop, item))
                return;
            let groupKey = (_b = prop.group) !== null && _b !== void 0 ? _b : "";
            if (groupedProps[groupKey] == null)
                groupedProps[groupKey] = [];
            groupedProps[groupKey].push(_);
        });
        let propertyGroupings = Object.keys(groupedProps).map(_ => {
            let group = Object.keys(groups).length && groups[_] ? groups[_] : { displayName: "Not grouped" }; // TODO: fuck the stupid typescript linter (ewi)
            return {
                groupKey: _,
                displayName: group.displayName,
                groupProps: Object.assign({}, group.groupProps),
                properties: groupedProps[_],
                validationError: groupValidation(groupedProps[_])
            };
        });
        setGroupings(propertyGroupings);
        setGroupsHandled(true);
    };
    const groupValidation = (properties) => {
        if (validationResults == null)
            return false;
        let error = false;
        properties === null || properties === void 0 ? void 0 : properties.forEach(_ => {
            const result = validationResults[_];
            if (result == null)
                return;
            Object.keys(result).forEach(r => {
                var res = result[r];
                if (res != null && !res.success)
                    error = true;
            });
        });
        return error;
    };
    const onPropertyChange = (property, value) => {
        var _a;
        let clone = Object.assign({}, item);
        clone[property] = value;
        // setItem(clone);
        // validateItem(clone, schema, ValidationEventType.Change, property);
        var onChanged = (_a = schema === null || schema === void 0 ? void 0 : schema.options.properties[property]) === null || _a === void 0 ? void 0 : _a.onChanged;
        if (onChanged != null) {
            let result = onChanged(clone);
            if (result === null || result === void 0 ? void 0 : result.refreshDynamicSchema) {
                loadSchema(clone);
            }
        }
        setItem(clone);
        validateItem(clone, schema, ValidationEventType.Change, property);
        if (props.onPropertyChange)
            props.onPropertyChange(clone, property, value);
    };
    const Spinner = formbuilder.formItemRender.loadingComponent();
    if (item == null)
        return null;
    if (schema == null)
        return React.createElement(Spinner, Object.assign({}, props.loadingProps));
    if (!groupsHandled)
        return null;
    const propertyRenderProps = {
        item: item,
        schema: schema.options,
        onChange: onPropertyChange,
        onBlur: (prop) => validateItem(item, schema, ValidationEventType.Blur, prop),
        validationResults: validationResults,
        validationResultPrefix: props.validationResultPrefix,
        propertyOverrides: props.propertyOverrides // this is merged with the schema, and shouldnt be part of the child items (ewi)
    };
    return (React.createElement("div", { className: "formbuilder-formcontainer", key: `${props.keyPrefix}-formbuilder-formcontainer` },
        React.createElement(GroupContainer, { groupings: groupings, key: `${props.keyPrefix}-formbuilder-groupcontainer` }, (_c = groupings === null || groupings === void 0 ? void 0 : groupings.map((_, idx) => {
            let children = formbuilder.formItemRender.properties(propertyRenderProps, _.properties);
            if (children === undefined)
                return null;
            const GroupRender = props.groupRender;
            return GroupRender ? React.createElement(GroupRender, { grouping: _, key: `group-${_.groupKey}-${idx}` }, children) : children;
        })) !== null && _c !== void 0 ? _c : formbuilder.formItemRender.properties(propertyRenderProps) // not defining groups here, defaults to all
        )));
});
//# sourceMappingURL=Form.js.map