var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import React, { useEffect, useImperativeHandle, useRef, useState } from 'react';
import { ValidationEventType } from '../models/validation/ValidationEventType';
import { Form } from './Form';
import { FormList } from './FormList';
import { formbuilder } from '../utils/FormBuilderInitializer';
import { schemaFromConfig } from '../utils/schema/schemaFromConfig';
// wrapper for handling both single item and list of items
// export const FormBuilder = forwardRef(<T extends IFormItem, FormBuilderRef>(props : IFormBuilderProps<T>, ref: FormBuilderRef) => {
export const FormBuilder = (props) => {
    var _a, _b;
    // verify that the formbuilder is set up correctly
    formbuilder.verify();
    const [item, setItem] = useState(props.item);
    const subFormRef = useRef();
    const [schema, setSchema] = useState(props.overrideSchema);
    useImperativeHandle(props.formRef, () => ({
        getItem: () => { var _a; return (_a = subFormRef.current) === null || _a === void 0 ? void 0 : _a.getItem(); },
        validateItem: () => { var _a, _b; return ((_a = subFormRef.current) === null || _a === void 0 ? void 0 : _a.validateItem) != null ? (_b = subFormRef.current) === null || _b === void 0 ? void 0 : _b.validateItem(ValidationEventType.Manual) : undefined; }
    }), []);
    const loadSchema = () => __awaiter(void 0, void 0, void 0, function* () {
        let schemaMerged = yield schemaFromConfig(props.schemaConfig, props.formItemConfigOverrides, props.propertyOverrides);
        setSchema(schemaMerged);
    });
    useEffect(() => {
        setItem(props.item);
    }, [props.item]);
    useEffect(() => {
        if (props.overrideSchema == null)
            loadSchema();
    }, [props.schemaConfig]);
    if (schema == null)
        return null;
    const getKey = () => {
        return `${props.keyPrefix}`;
    };
    return (React.createElement("div", { className: "formbuilder-rootcontainer", key: `${props.keyPrefix}-formbuilder-container` }, (Array.isArray(props.item) || props.listProps != null)
        ? React.createElement(FormList, Object.assign({ key: getKey(), ref: subFormRef, items: item, schema: schema }, props, { listProps: (_a = props.listProps) !== null && _a !== void 0 ? _a : schema.options.listOptions, keyPrefix: `${props.keyPrefix}-list`, readOnly: ((_b = props.propertyOverrides) === null || _b === void 0 ? void 0 : _b.disabledProps) === true }))
        : React.createElement(Form, Object.assign({ key: getKey(), ref: subFormRef, schema: schema, item: item, onPropertyChange: (item, prop, value) => { var _a; return ((_a = props.singleItemProps) === null || _a === void 0 ? void 0 : _a.onPropertyChange) == null ? () => { } : props.singleItemProps.onPropertyChange(item, prop, value); } }, props, { keyPrefix: getKey() }))));
};
//# sourceMappingURL=FormBuilder.js.map